;################################################################## BASSER.ASL
; Purpose.....: Send/Receive Basic program and Basic work area over serial
;		link. Serial parameters are 4800,N,8,1.
; Target......: MPF-1P
; Assembler...: ASL V1.42 Beta
; Command Line:	ASL -listline-prefix %04a -L -U -olist <name>.prn <name>.asl
; PsPadParser.: > > > %F(%L):%C: error:
;#############################################################################

;********************************************************************* history
;=========================================================== V01.00:01.09.2024
; Created
;***************************************************************** end history

;*********************************************************************** notes
;=================================================================== operation
; This program in intended to be CALLed from within Basic
; CALL 61440/46144: (debug/ROM) Save source and work area
; CALL 61443/46147: (debug/ROM) Load source and work area
; After Save/Load, it returns to Basic warm boot at 2020
;******************************************************************* end notes

;***************************************************************** definitions
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++ assembler directives
		CPU	Z80		; use Z80 CPU
		PAGE	0		; no formfeeds in listing file
		
;+++++++++++++++++++++++++++++++++++ hardware, monitor and library definitions
		LISTING	OFF
		INCLUDE	../LIBRARY/MPF1P.DEF
		LISTING	ON
		
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ build addresses
wBSCS:		EQU	0B440H		; code (B440-B48F if IOM-ROM)
wBSDS:		EQU	0FAF8H		; data below Basic work area

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ Basic addresses
wBSWBT:		EQU	02020H		; Basic warm boot entry
wBSSRC:		EQU	0D800H		; start of Basic source code
wBSWST:		EQU	0FB00H		; start of Basic work area (assumed)
wBSWEN:		EQU	0FEFFH		; end of Basic work area
wBSWZ:		EQU	wBSWEN-wBSWST	; size of work area

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ variables
wBSVS:		EQU	wBSDS		; variables start address
wBSVE:		EQU	wBSVS		; variables end address

;======================================== check overlap with library variables
		IF	wBSVE > dwLVST && MomPass = 2
		WARNING	"Variables overlap with library:\{wBSVE} >> \{dwLVST}"
		ENDIF
;************************************************************* end definitions

;************************************************************** implementation
		ASEG
		ORG	wBSCS

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ main program
;======================================================================== BS()
; Purpose.....: Main program
; Labels......: B)asic S)erial
;=============================================================================
BS:		JP	BSS
		JP	BSL
;==================================================================== eoM BS()
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end main program

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ save/load routines
;======================================================================= BSS()
; Purpose.....: Send Basic source and work area
; Labels......: B)asic S)erial S)save
;=============================================================================
BSS:		CALL	BSINIT		; init library and USART

;----------------------------------------------------------- send Basic source
		LD	A,'S'
		CALL	BSDP		; display saving pattern

		LD	E,0		; E= NULLs counter
		LD	HL,wBSSRC	; HL= address of source code

BSSSL:		CALL	BSTX		; check for abort request, send byte
		JR	C,BSSX		; shift and control pressed, exit

;----------------------------------------- terminates on two consecutive NULLs
		LD	A,(HL)		; just sent byte
		INC	HL		; bump address pointer
		OR	A		; is it zero?
		JR	Z,BSSSBN	; yes, check for two NULLs
		LD	E,0		; no, clear nulls counter
		JR	BSSSL		; and send next
BSSSBN:		INC	E		; bump nulls counter
		BIT	1,E		; are there two?
		JR	Z,BSSSL		; no, send next

;-------------------------------------------------------- send Basic work area
		LD	HL,wBSWST	; HL= work area start
		LD	DE,wBSWZ	; DE= work area size

BSSWL:		CALL	BSTX		; check for abort request, send byte
		JR	C,BSSX		; shift and control pressed, exit
		LD	A,D		; check if all done
		OR	E
		JR	Z,BSSX		; all done, exit
		DEC	DE		; dec byte counter
		INC	HL		; bump address pointer
		JR	BSSWL		; send next

BSSX:		JP	wBSWBT		; back to Basic warm boot
;=================================================================== eoM BSS()

;======================================================================= BSL()
; Purpose.....: Receive Basic source and work area
; Labels......: B)asic S)erial L)oad
;=============================================================================
BSL:		CALL	BSINIT		; init library and USART

;----------------------------------------------------------- send Basic source
		LD	A,'L'
		CALL	BSDP		; display loading pattern

		SCF			; set RTS
		CALL	dfLURTS		; active
		LD	E,0		; E= NULLs counter
		LD	HL,wBSSRC	; HL= address of source code

BSLSL:		CALL	BSRX		; check abort, receive byte
		JR	C,BSLX		; shift and control pressed, exit

;----------------------------------------- terminates on two consecutive NULLs
		LD	(HL),A		; store received byte
		INC	HL		; bump address pointer
		OR	A		; is receive byte zero?
		JR	Z,BSLSBN	; yes, check for two NULLs
		LD	E,0		; no, clear nulls counter
		JR	BSLSL		; and receive next
BSLSBN:		INC	E		; bump nulls counter
		BIT	1,E		; are there two?
		JR	Z,BSLSL		; no, receive next

;----------------------------------------------------- receive Basic work area
		LD	HL,wBSWST	; HL= work area start
		LD	DE,wBSWZ	; DE= work area size

BSLWL:		CALL	BSRX		; check abort request, receive byte
		JR	C,BSLX		; shift and control pressed, exit

		LD	(HL),A		; store received byte
		LD	A,D		; check if all done
		OR	E
		JR	Z,BSLX		; all done, exit
		DEC	DE		; dec byte counter
		INC	HL		; bump address pointer
		JR	BSLWL		; receive next

BSLX:		XOR	A		; set RTS
		CALL	dfLURTS		; inactive
		JP	wBSWBT		; back to Basic warm boot
;=================================================================== eoF BSL()
;++++++++++++++++++++++++++++++++++++++++++++++++++++++ end save/load routines

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ subroutines
;==================================================================== BSINIT()
; Purpose.....: Initialize library and USART
; Registers...:	None saved
; Labels......: B)asic S)erial I)nit
;=============================================================================
BSINIT:		LD	HL,(2001H)	; HL= bytes 2&3 of Basic ROM
		LD	A,0B9H		; check if ROM present
		CP	L		; if not, return to monitor
		JP	NZ,dwMRET	; used if not called from
		LD	A,009H		; within Basic
		CP	H
		JP	NZ,dwMRET

		CALL	dfLLS		; initialize library variables

;----------------------------------------------------------------- setup USART
		LD	B,dbUB4K	; baudrate 4800
		LD	C,dbU8N1	; 8 data, no parity, 1 stop
		CALL	dfLUS		; setup USART
		RET
;================================================================ eoF BSINIT()

;====================================================================== BSRX()
; Purpose.....: Check keyboard abort and receive a byte
; Output......: A= received byte, undefined if CY=1
;		CY: 1= shift & control were pressed
; Registers...:	None saved
; Labels......: B)asic S)erial R)eceive
;=============================================================================
BSRX:		CALL	dfLKSAC		; check for abort request
		JR	C,BSRXX		; shift and control pressed, exit
		CALL	dfLURB		; try to receive a byte
		JR	Z,BSRX		; none ready, wait
BSRXX:		RET
;================================================================== eoF BSRX()

;====================================================================== BSTX()
; Purpose.....: Check keyboard abort and send a byte
; Input.......: (HL): Byte to send
; Output......: CY:   1= shift & control were pressed
; Registers...:	None saved
; Labels......: B)asic S)erial T)ransmit
;=============================================================================
BSTX:		CALL	dfLKSAC		; check for abort request
		JR	C,BSTXX		; shift and control pressed, exit
		LD	A,(HL)		; get next byte to send
		CALL	dfLUTB		; try to send the byte
		JR	C,BSTX		; not sent, retry
BSTXX:		RET
;================================================================== eoF BSTX()

;====================================================================== BSDP()
; Purpose.....:	Display static pattern in the center of the display to
;		signal progress.
; Input.......: A: Pattern character to display
; Registers...:	AF destroyed
; Labels......: B)asic S)ave D)isplay P)attern
;=============================================================================
BSDP:		PUSH	DE
		PUSH	BC

		LD	D,00011111B	; digits 8-16
		LD	E,10000000B	; digits 1-8
		LD	C,00000000B	; digits 17-20
		LD	B,A		; save pattern
		CALL	dfLDP		; display the pattern

		POP	BC
		POP	DE
		RET
;================================================================== eoF RBDP()
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end subroutines

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ size info
wBSCSE:		EQU	$
		IF	MomPass = 2
		MESSAGE "Code from \{wBSCS} to \{wBSCSE} Size \{wBSCSE-wBSCS}"
		MESSAGE "Data from \{wBSDS} to \{wBSVE} Size \{wBSVE-wBSDS}"
		MESSAGE "     remaining \{dwLVST-wBSVE}"
		ENDIF
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ end size info

;############################################################# EOF BASSAVE.ASL
		END
